@@ -300,6 +300,8 @@ GRN_API grn_rc grn_unset_variable(const char *name,
 GRN_API int grn_get_lock_timeout(void);
 GRN_API grn_rc grn_set_lock_timeout(int timeout);
 
+GRN_API size_t grn_get_memory_map_size(void);
+
 /* grn_encoding */
 
 GRN_API const char *grn_encoding_to_string(grn_encoding encoding);
@@ -382,6 +384,16 @@ typedef uint32_t grn_column_flags;
 #define GRN_OBJ_INDEX_LARGE            (0x01<<18)
 #define GRN_OBJ_WEIGHT_FLOAT32         (0x01<<19)
 
+#define GRN_OBJ_MISSING_MASK           (0x03<<20)
+#define GRN_OBJ_MISSING_ADD            (0x00<<20)
+#define GRN_OBJ_MISSING_IGNORE         (0x01<<20)
+#define GRN_OBJ_MISSING_NIL            (0x02<<20)
+
+#define GRN_OBJ_INVALID_MASK           (0x03<<22)
+#define GRN_OBJ_INVALID_ERROR          (0x00<<22)
+#define GRN_OBJ_INVALID_WARN           (0x01<<22)
+#define GRN_OBJ_INVALID_IGNORE         (0x02<<22)
+
 /* flags only for grn_table_flags and grn_column_flags */
 
 /* GRN_COLUMN_INDEX only uses this for now */
@@ -625,7 +637,7 @@ typedef enum {
 } grn_operator;
 
 GRN_API grn_obj *grn_obj_column(grn_ctx *ctx, grn_obj *table,
-                                const char *name, unsigned int name_size);
+                                const char *name, uint32_t name_size);
 
 /*-------------------------------------------------------------
  * API for db, table and/or column
@@ -720,7 +732,7 @@ GRN_API grn_rc grn_column_rename(grn_ctx *ctx, grn_obj *column,
                                  const char *name, unsigned int name_size);
 
 GRN_API grn_rc grn_obj_close(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_rc grn_obj_reinit(grn_ctx *ctx, grn_obj *obj, grn_id domain, unsigned char flags);
+GRN_API grn_rc grn_obj_reinit(grn_ctx *ctx, grn_obj *obj, grn_id domain, uint8_t flags);
 /* On non reference count mode (default):
  * This closes the following objects immediately:
  *
@@ -890,7 +902,7 @@ GRN_API grn_rc grn_column_get_all_index_columns(grn_ctx *ctx,
                                                 grn_obj *column,
                                                 grn_obj *index_columns);
 
-GRN_API grn_rc grn_obj_delete_by_id(grn_ctx *ctx, grn_obj *db, grn_id id, grn_bool removep);
+GRN_API grn_rc grn_obj_delete_by_id(grn_ctx *ctx, grn_obj *db, grn_id id, bool remove_p);
 GRN_API grn_rc grn_obj_path_by_id(grn_ctx *ctx, grn_obj *db, grn_id id, char *buffer);
 
 /* query & snippet */
@@ -1149,7 +1161,7 @@ GRN_API off_t grn_default_query_logger_get_rotate_threshold_size(void);
       if (GRN_BULK_OUTP(_body)) {\
         (_body)->u.b.curr = (_body)->u.b.head;\
       } else {\
-        (_body)->header.flags &= ~GRN_BULK_BUFSIZE_MAX;\
+        (_body)->header.flags &= (grn_obj_flags)(~GRN_BULK_BUFSIZE_MAX);\
       }\
     }\
     (bulk)->u.v.n_sections = 0;\
@@ -1157,7 +1169,7 @@ GRN_API off_t grn_default_query_logger_get_rotate_threshold_size(void);
     if (GRN_BULK_OUTP(bulk)) {\
       (bulk)->u.b.curr = (bulk)->u.b.head;\
     } else {\
-      (bulk)->header.flags &= ~GRN_BULK_BUFSIZE_MAX;\
+      (bulk)->header.flags &= (grn_obj_flags)(~GRN_BULK_BUFSIZE_MAX);\
     }\
   }\
 } while (0)
@@ -1165,7 +1177,7 @@ GRN_API off_t grn_default_query_logger_get_rotate_threshold_size(void);
   if (GRN_BULK_OUTP(buf)) {\
     (buf)->u.b.curr = (char *)(p);\
   } else {\
-    (buf)->header.flags = (char *)(p) - GRN_BULK_HEAD(buf);\
+    (buf)->header.flags = (grn_obj_flags)((char *)(p) - GRN_BULK_HEAD(buf));\
   }\
 } while (0)
 #define GRN_BULK_INCR_LEN(bulk,len) do {\
@@ -1554,7 +1566,8 @@ GRN_API void grn_ctx_recv_handler_set(grn_ctx *,
 
 #define GRN_BULK_POP(obj, value, type, default) do {\
   if (GRN_BULK_VSIZE(obj) >= sizeof(type)) {\
-    GRN_BULK_INCR_LEN((obj), -(sizeof(type)));\
+    ssize_t value_size = (ssize_t)(sizeof(type));\
+    GRN_BULK_INCR_LEN((obj), -value_size);\
     value = *(type *)(GRN_BULK_CURR(obj));\
   } else {\
     value = default;\
@@ -1612,7 +1625,7 @@ GRN_API grn_rc grn_load(grn_ctx *ctx, grn_content_type input_type,
 
 GRN_API grn_rc grn_ctx_connect(grn_ctx *ctx, const char *host, int port, int flags);
 GRN_API unsigned int grn_ctx_send(grn_ctx *ctx, const char *str, unsigned int str_len, int flags);
-GRN_API unsigned int grn_ctx_recv(grn_ctx *ctx, char **str, unsigned int *str_len, int *flags);
+GRN_API grn_rc grn_ctx_recv(grn_ctx *ctx, char **str, unsigned int *str_len, int *flags);
 
 typedef struct _grn_ctx_info grn_ctx_info;
 
