@@ -17,8 +17,10 @@
 */
 
 #include "grn.h"
-#include "grn_db.h"
 #include "grn_arrow.h"
+#include "grn_cast.h"
+#include "grn_column.h"
+#include "grn_db.h"
 #include "grn_output.h"
 
 #ifdef GRN_WITH_APACHE_ARROW
@@ -68,6 +70,12 @@ namespace grnarrow {
     }
   }
 
+  arrow::Status check(grn_ctx *ctx,
+                      grn_rc rc,
+                      std::string context) {
+    return check(ctx, rc, context.c_str());
+  }
+
   bool check(grn_ctx *ctx,
              const arrow::Status &status,
              const char *context) {
@@ -234,45 +242,6 @@ namespace grnarrow {
     return dictionary_data_type->value_type();
   }
 
-  class ArrayBuilderResetFullVisitor : public arrow::TypeVisitor {
-    // We need to execute ResetFull() with an appropriate dictionary length
-    // in order to avoid memory exhaustion and so on.
-    // - Dictionary data in DictionaryBuilder are held in memory until
-    //   ResetFull() is executed.
-    // - Some Apache Arrow implementations (clients) have a 2GB limitation on
-    //   the value size of array: C#.
-  public:
-    ArrayBuilderResetFullVisitor(grn_ctx *ctx,
-                                 arrow::ArrayBuilder *builder)
-      : ctx_(ctx),
-      builder_(builder) {
-    }
-
-    arrow::Status Visit(const arrow::DictionaryType &type) override
-    {
-      const int64_t dictionary_length_threshold = 10000;
-
-      // The value type of Dictionary is always string for now.
-      auto builder = static_cast<arrow::StringDictionaryBuilder *>(builder_);
-      if (builder->dictionary_length() > dictionary_length_threshold) {
-        builder->ResetFull();
-      }
-      return arrow::Status::OK();
-    }
-
-    arrow::Status Visit(const arrow::ListType &type) override
-    {
-      auto builder = static_cast<arrow::ListBuilder *>(builder_);
-      auto value_builder = builder->value_builder();
-      ArrayBuilderResetFullVisitor value_visitor(ctx_, value_builder);
-      return value_builder->type()->Accept(&value_visitor);
-    }
-
-  private:
-    grn_ctx *ctx_;
-    arrow::ArrayBuilder *builder_;
-  };
-
   class RecordAddVisitor : public arrow::ArrayVisitor {
   public:
     RecordAddVisitor(grn_ctx *ctx,
@@ -535,6 +504,7 @@ namespace grnarrow {
         bulk_(bulk),
         index_(index),
         buffer_(),
+        loaded_value_(nullptr),
         object_cache_(object_cache) {
       GRN_VOID_INIT(&buffer_);
     }
@@ -688,9 +658,11 @@ namespace grnarrow {
         {
           grn_obj sub_buffer;
           if (grn_type_id_is_text_family(ctx_, bulk_->header.domain)) {
-            GRN_TEXT_INIT(&sub_buffer, 0);
+            GRN_TEXT_INIT(&sub_buffer, GRN_OBJ_VECTOR);
           } else {
-            GRN_VALUE_FIX_SIZE_INIT(&sub_buffer, 0, bulk_->header.domain);
+            GRN_VALUE_FIX_SIZE_INIT(&sub_buffer,
+                                    GRN_OBJ_VECTOR,
+                                    bulk_->header.domain);
           }
           for (int64_t i = 0; i < value_array->length(); ++i) {
             GRN_BULK_REWIND(&sub_buffer);
@@ -703,22 +675,30 @@ namespace grnarrow {
             if (ctx_->rc != GRN_SUCCESS) {
               continue;
             }
-            if (grn_type_id_is_text_family(ctx_, bulk_->header.domain)) {
-              uint32_t weight = 0;
-              grn_id domain = GRN_DB_SHORT_TEXT;
-              const auto sub_buffer_size =
-                static_cast<unsigned int>(GRN_TEXT_LEN(&sub_buffer));
-              grn_vector_add_element(ctx_,
-                                     bulk_,
-                                     GRN_TEXT_VALUE(&sub_buffer),
-                                     sub_buffer_size,
-                                     weight,
-                                     domain);
+            auto loaded_value = sub_visitor.loaded_value();
+            if (grn_obj_is_vector(ctx_, bulk_)) {
+              auto original_value = sub_visitor.original_value();
+              if (grn_obj_is_text_family_bulk(ctx_, original_value) &&
+                  GRN_TEXT_LEN(original_value) == 0 &&
+                  grn_type_id_is_text_family(ctx_, bulk_->header.domain)) {
+                grn_vector_add_element_float(ctx_,
+                                             bulk_,
+                                             NULL,
+                                             0,
+                                             0.0,
+                                             bulk_->header.domain);
+              } else {
+                grn_vector_copy(ctx_, loaded_value, bulk_);
+              }
             } else {
-              grn_bulk_write(ctx_,
-                             bulk_,
-                             GRN_BULK_HEAD(&sub_buffer),
-                             GRN_BULK_VSIZE(&sub_buffer));
+              auto original_value = sub_visitor.original_value();
+              if (grn_obj_is_text_family_bulk(ctx_, original_value) &&
+                  GRN_TEXT_LEN(original_value) == 0 &&
+                  grn_id_maybe_table(ctx_, bulk_->header.domain)) {
+                grn_uvector_add_element_record(ctx_, bulk_, GRN_ID_NIL, 0.0);
+              } else {
+                grn_uvector_copy(ctx_, loaded_value, bulk_);
+              }
             }
           }
           GRN_OBJ_FIN(ctx_, &sub_buffer);
@@ -757,40 +737,43 @@ namespace grnarrow {
       const grn_id domain = bulk_->header.domain;
       const auto raw_value = value.data();
       const auto raw_value_size = value.size();
+      grn_obj *value_bulk;
       grn_obj value_buffer;
       if (grn_type_id_is_text_family(ctx_, domain)) {
         GRN_TEXT_INIT(&value_buffer, GRN_OBJ_DO_SHALLOW_COPY);
         GRN_TEXT_SET(ctx_, &value_buffer, raw_value, raw_value_size);
+        value_bulk = &value_buffer;
       } else {
         grn_obj raw_value_buffer;
         GRN_TEXT_INIT(&raw_value_buffer, GRN_OBJ_DO_SHALLOW_COPY);
         GRN_TEXT_SET(ctx_, &raw_value_buffer, raw_value, raw_value_size);
         GRN_VALUE_FIX_SIZE_INIT(&value_buffer, 0, domain);
-        if (grn_obj_cast(ctx_,
-                         &raw_value_buffer,
-                         &value_buffer,
-                         true) != GRN_SUCCESS) {
-          auto ctx = ctx_;
-          auto domain_object = (*object_cache_)[domain];
-          ERR_CAST(grn_column_, domain_object, &raw_value_buffer);
+        value_bulk = grn_column_cast_value(ctx_,
+                                           grn_column_,
+                                           &raw_value_buffer,
+                                           &value_buffer,
+                                           GRN_OBJ_SET);
+        if (ctx_->rc != GRN_SUCCESS &&
+            (grn_obj_is_vector(ctx_, bulk_) ||
+             grn_obj_is_uvector(ctx_, bulk_))) {
+          ERRCLR(ctx_);
         }
+        GRN_OBJ_FIN(ctx_, &raw_value_buffer);
       }
-      if (GRN_BULK_VSIZE(&value_buffer) > 0) {
+      if (value_bulk && GRN_BULK_VSIZE(value_bulk) > 0) {
         if (grn_type_id_is_text_family(ctx_, domain)) {
-          const auto value_buffer_size =
-            static_cast<unsigned int>(GRN_BULK_VSIZE(&value_buffer));
+          const auto value_bulk_size =
+            static_cast<unsigned int>(GRN_BULK_VSIZE(value_bulk));
           grn_vector_add_element_float(ctx_,
                                        bulk_,
-                                       GRN_BULK_HEAD(&value_buffer),
-                                       value_buffer_size,
+                                       GRN_BULK_HEAD(value_bulk),
+                                       value_bulk_size,
                                        weight,
                                        domain);
         } else {
           // TODO: Support weight vector for number such as Int64
-          grn_uvector_add_element_record(ctx_,
-                                         bulk_,
-                                         GRN_RECORD_VALUE(&value_buffer),
-                                         weight);
+          auto id = grn_uvector_get_element_record(ctx_, value_bulk, 0, nullptr);
+          grn_uvector_add_element_record(ctx_, bulk_, id, weight);
         }
       }
       GRN_OBJ_FIN(ctx_, &value_buffer);
@@ -801,24 +784,51 @@ namespace grnarrow {
       return &buffer_;
     }
 
+    grn_obj *loaded_value() {
+      if (loaded_value_) {
+        return loaded_value_;
+      }
+      return bulk_;
+    }
+
   private:
     grn_ctx *ctx_;
     grn_obj *grn_column_;
     grn_obj *bulk_;
     int64_t index_;
     grn_obj buffer_;
+    grn_obj *loaded_value_;
     ObjectCache *object_cache_;
 
     template <typename LoadBulk>
     arrow::Status
     load_value(LoadBulk load_bulk) {
       load_bulk();
-      if (bulk_->header.domain != GRN_DB_VOID) {
+      loaded_value_ = &buffer_;
+      if (bulk_->header.domain == GRN_DB_VOID) {
+        return arrow::Status::OK();
+      }
+      if (!grn_column_) {
+        return arrow::Status::OK();
+      }
+      if (grn_obj_is_accessor(ctx_, grn_column_)) {
         if (grn_obj_cast(ctx_, &buffer_, bulk_, true) != GRN_SUCCESS) {
           auto ctx = ctx_;
           auto range = (*object_cache_)[bulk_->header.domain];
           ERR_CAST(grn_column_, range, &buffer_);
         }
+        loaded_value_ = bulk_;
+      } else {
+        loaded_value_ = grn_column_cast_value(ctx_,
+                                              grn_column_,
+                                              &buffer_,
+                                              bulk_,
+                                              GRN_OBJ_SET);
+        if (ctx_->rc != GRN_SUCCESS &&
+            (grn_obj_is_vector(ctx_, bulk_) ||
+             grn_obj_is_uvector(ctx_, bulk_))) {
+          ERRCLR(ctx_);
+        }
       }
       return arrow::Status::OK();
     }
@@ -874,13 +884,16 @@ namespace grnarrow {
                               (*object_cache_)[arrow_type_id]);
         }
       }
-      if (grn_type_id_is_text_family(ctx_, arrow_type_id)) {
-        GRN_VALUE_VAR_SIZE_INIT(&buffer_, flags, arrow_type_id);
+      grn_id range_id = GRN_ID_NIL;
+      if (grn_column_) {
+        grn_obj_get_range_info(ctx, grn_column_, &range_id, &flags);
+      }
+      if (range_id == GRN_ID_NIL) {
+        range_id = arrow_type_id;
+      }
+      if (grn_type_id_is_text_family(ctx_, range_id)) {
+        GRN_VALUE_VAR_SIZE_INIT(&buffer_, flags, range_id);
       } else {
-        grn_id range_id = GRN_ID_NIL;
-        if (grn_column_) {
-          range_id = grn_obj_get_range(ctx, grn_column_);
-        }
         GRN_VALUE_FIX_SIZE_INIT(&buffer_, flags, range_id);
         if (flags & GRN_OBJ_WITH_WEIGHT) {
           buffer_.header.flags |= GRN_OBJ_WITH_WEIGHT;
@@ -1081,13 +1094,14 @@ namespace grnarrow {
           continue;
         }
 
-        grn_obj_set_value(ctx_, grn_column_,record_id, &buffer_, GRN_OBJ_SET);
+        auto value = visitor.loaded_value();
+        grn_obj_set_value(ctx_, grn_column_, record_id, value, GRN_OBJ_SET);
         if (ctx_->rc != GRN_SUCCESS) {
           if (grn_loader_) {
             grn_loader_add_record_data data;
             init_grn_loader_add_record_data(&data,
                                             record_id,
-                                            &buffer_);
+                                            value);
             grn_loader_on_column_set(ctx_, grn_loader_, &data);
           }
         }
@@ -1186,7 +1200,8 @@ namespace grnarrow {
       : ctx_(ctx),
         grn_table_(grn_table),
         grn_columns_(grn_columns),
-        object_cache_(ctx_) {
+        object_cache_(ctx_),
+        tag_("[arrow][dump]") {
     }
 
     ~FileDumper() {
@@ -1224,7 +1239,7 @@ namespace grnarrow {
 
       if (!check(ctx_,
                  writer_result,
-                 "[arrow][dump] failed to create file format writer")) {
+                 tag_ + " failed to create file format writer")) {
         return ctx_->rc;
       }
       auto writer = *writer_result;
@@ -1236,15 +1251,21 @@ namespace grnarrow {
         if (ids.size() == n_records_per_batch) {
           write_record_batch(ids, schema, writer);
           ids.clear();
+          if (ctx_->rc != GRN_SUCCESS) {
+            break;
+          }
         }
       } GRN_TABLE_EACH_END(ctx_, table_cursor);
       if (!ids.empty()) {
         write_record_batch(ids, schema, writer);
       }
+      if (ctx_->rc != GRN_SUCCESS) {
+        return ctx_->rc;
+      }
       auto status = writer->Close();
       if (!check(ctx_,
                  status,
-                 "[arrow][dump] failed to close writer file format writer")) {
+                 tag_ + " failed to close writer file format writer")) {
         return ctx_->rc;
       }
 
@@ -1256,6 +1277,7 @@ namespace grnarrow {
     grn_obj *grn_table_;
     grn_obj *grn_columns_;
     ObjectCache object_cache_;
+    std::string tag_;
 
     void write_record_batch(std::vector<grn_id> &ids,
                             std::shared_ptr<arrow::Schema> &schema,
@@ -1322,7 +1344,9 @@ namespace grnarrow {
       }
 
       auto record_batch = arrow::RecordBatch::Make(schema, ids.size(), columns);
-      writer->WriteRecordBatch(*record_batch);
+      check(ctx_,
+            writer->WriteRecordBatch(*record_batch),
+            tag_ + " failed to write record batch");
     }
 
     arrow::Status build_boolean_array(std::vector<grn_id> &ids,
@@ -1607,7 +1631,8 @@ namespace grnarrow {
         grn_loader_(loader),
         decoder_(std::shared_ptr<StreamLoader>(this, [](void*) {})),
         buffer_(nullptr),
-        object_cache_(ctx_) {
+        object_cache_(ctx_),
+        tag_("[arrow][stream-loader]") {
     }
 
     grn_rc consume(const char *data, size_t data_size) {
@@ -1619,10 +1644,7 @@ namespace grnarrow {
     }
 
     arrow::Status OnRecordBatchDecoded(std::shared_ptr<arrow::RecordBatch> record_batch) override {
-      process_record_batch(std::move(record_batch));
-      return check(ctx_,
-                   ctx_->rc,
-                   "[arrow][stream-loader][consume][record-batch-decoded]");
+      return process_record_batch(std::move(record_batch));
     }
 
   private:
@@ -1631,8 +1653,7 @@ namespace grnarrow {
         auto buffer = arrow::AllocateResizableBuffer(0);
         if (!check(ctx_,
                    buffer,
-                   "[arrow][stream-loader][consume] "
-                   "failed to allocate buffer")) {
+                   tag_ + "[consume] failed to allocate buffer")) {
           return ctx_->rc;
         }
         buffer_ = std::move(*buffer);
@@ -1641,7 +1662,7 @@ namespace grnarrow {
       auto current_buffer_size = buffer_->size();
       if (!check(ctx_,
                  buffer_->Resize(current_buffer_size + data_size),
-                 "[arrow][stream-loader][consume] failed to resize buffer")) {
+                 tag_ + "[consume] failed to resize buffer")) {
         return ctx_->rc;
       }
       grn_memcpy(buffer_->mutable_data() + current_buffer_size,
@@ -1655,13 +1676,14 @@ namespace grnarrow {
       std::shared_ptr<arrow::Buffer> chunk(buffer_.release());
       if (!check(ctx_,
                  decoder_.Consume(chunk),
-                 "[arrow][stream-loader][consume] failed to consume")) {
+                 tag_ + "[consume] failed to consume")) {
         return ctx_->rc;
       }
       return ctx_->rc;
     }
 
-    void process_record_batch(std::shared_ptr<arrow::RecordBatch> record_batch) {
+    arrow::Status process_record_batch(
+      std::shared_ptr<arrow::RecordBatch> record_batch) {
       auto grn_table = grn_loader_->table;
       const auto &key_column = record_batch->GetColumnByName("_key");
       const auto &id_column = record_batch->GetColumnByName("_id");
@@ -1672,13 +1694,13 @@ namespace grnarrow {
                                  grn_loader_,
                                  &record_ids,
                                  true);
-        key_column->Accept(&visitor);
+        ARROW_RETURN_NOT_OK(key_column->Accept(&visitor));
       } else if (id_column) {
         RecordAddVisitor visitor(ctx_,
                                  grn_loader_,
                                  &record_ids,
                                  false);
-        id_column->Accept(&visitor);
+        ARROW_RETURN_NOT_OK(id_column->Accept(&visitor));
       } else {
         for (int64_t i = 0; i < n_records; ++i) {
           const auto record_id = grn_table_add(ctx_, grn_table, NULL, 0, NULL);
@@ -1700,7 +1722,7 @@ namespace grnarrow {
                                   schema->field(i),
                                   record_ids.data(),
                                   &object_cache_);
-        column->Accept(&visitor);
+        ARROW_RETURN_NOT_OK(column->Accept(&visitor));
       }
       for (const auto record_id : record_ids) {
         if (record_id == GRN_ID_NIL) {
@@ -1708,6 +1730,9 @@ namespace grnarrow {
         }
         grn_loader_apply_each(ctx_, grn_loader_, record_id);
       }
+      return check(ctx_,
+                   ctx_->rc,
+                   tag_ + "[consume][record-batch-decoded]");
     }
 
     grn_ctx *ctx_;
@@ -1715,6 +1740,7 @@ namespace grnarrow {
     arrow::ipc::StreamDecoder decoder_;
     std::unique_ptr<arrow::ResizableBuffer> buffer_;
     ObjectCache object_cache_;
+    std::string tag_;
   };
 
   class BulkOutputStream : public arrow::io::OutputStream {
@@ -1787,15 +1813,16 @@ namespace grnarrow {
         record_batch_builder_(),
         n_records_(0),
         current_column_index_(0),
-        object_cache_(ctx_) {
+        object_cache_(ctx_),
+        tag_("[arrow][stream-writer]") {
     }
 
     ~StreamWriter() {
       flush();
       if (writer_) {
-        writer_->Close();
+        std::ignore = writer_->Close();
       }
-      output_.Close();
+      std::ignore = output_.Close();
     }
 
     void add_metadata(const char *key, const char *value) {
@@ -1807,7 +1834,8 @@ namespace grnarrow {
         check(ctx_,
               status,
               context <<
-              "[arrow][stream-writer][add-meatadata] " <<
+              tag_ <<
+              "[add-meatadata] " <<
               "failed to add metadata: <" <<
               key <<
               ">: <" <<
@@ -1824,8 +1852,8 @@ namespace grnarrow {
         GRN_TEXT_INIT(&inspected, 0);
         grn_inspect(ctx_, &inspected, column);
         ERR(GRN_FUNCTION_NOT_IMPLEMENTED,
-            "[arrow][stream-writer][add-field] "
-            "unsupported column: <%.*s>",
+            "%s[add-field] unsupported column: <%.*s>",
+            tag_.c_str(),
             (int)GRN_TEXT_LEN(&inspected),
             GRN_TEXT_VALUE(&inspected));
         GRN_OBJ_FIN(ctx_, &inspected);
@@ -1838,7 +1866,8 @@ namespace grnarrow {
         check(ctx_,
               status,
               context <<
-              "[arrow][stream-writer][add-field] " <<
+              tag_ <<
+              "[add-field] " <<
               "failed to add field: <" <<
               field->ToString() <<
               ">");
@@ -1846,38 +1875,34 @@ namespace grnarrow {
     }
 
     void write_schema() {
-      auto schema = schema_builder_.Finish();
+      auto schema_result = schema_builder_.Finish();
       if (!check(ctx_,
-                 schema,
-                 "[arrow][stream-writer][write-schema] "
-                 "failed to create schema")) {
+                 schema_result,
+                 tag_ + "[write-schema] failed to create schema")) {
           return;
       }
       schema_builder_.Reset();
-      schema_ = *schema;
+      schema_ = *schema_result;
 
       auto option = arrow::ipc::IpcWriteOptions::Defaults();
       option.emit_dictionary_deltas = true;
-      auto writer = arrow::ipc::MakeStreamWriter(&output_, schema_, option);
+      auto writer_result =
+        arrow::ipc::MakeStreamWriter(&output_, schema_, option);
 
       if (!check(ctx_,
-                 writer,
-                 "[arrow][stream-writer][write-schema] "
-                 "failed to create writer")) {
+                 writer_result,
+                 tag_ + "[write-schema] failed to create writer")) {
         return;
       }
-      writer_ = *writer;
+      writer_ = *writer_result;
 
       auto status =
         arrow::RecordBatchBuilder::Make(schema_,
                                         arrow::default_memory_pool(),
                                         &record_batch_builder_);
-      if (!check(ctx_,
-                 status,
-                 "[arrow][stream-writer][write-schema] "
-                 "failed to create record batch builder")) {
-        return;
-      }
+      check(ctx_,
+             status,
+            tag_ + "[write-schema] failed to create record batch builder");
     }
 
     void open_record() {
@@ -1903,7 +1928,8 @@ namespace grnarrow {
       check(ctx_,
             status,
             context <<
-            "[arrow][stream-writer][add-column][string] " <<
+            tag_ <<
+            "[add-column][string] " <<
             "failed to add a column value: <" <<
             arrow::util::string_view(value, value_length) <<
             ">");
@@ -1921,7 +1947,8 @@ namespace grnarrow {
       check(ctx_,
             status,
             context <<
-            "[arrow][stream-writer][add-column][int8] " <<
+            tag_ <<
+            "[add-column][int8] " <<
             "failed to add a column value: <" << value << ">");
     }
 
@@ -1937,7 +1964,8 @@ namespace grnarrow {
       check(ctx_,
             status,
             context <<
-            "[arrow][stream-writer][add-column][int32] " <<
+            tag_ <<
+            "[add-column][int32] " <<
             "failed to add a column value: <" << value << ">");
     }
 
@@ -1953,7 +1981,8 @@ namespace grnarrow {
       check(ctx_,
             status,
             context <<
-            "[arrow][stream-writer][add-column][uint32] " <<
+            tag_ <<
+            "[add-column][uint32] " <<
             "failed to add a column value: <" << value << ">");
     }
 
@@ -1969,7 +1998,8 @@ namespace grnarrow {
       check(ctx_,
             status,
             context <<
-            "[arrow][stream-writer][add-column][int64] " <<
+            tag_ <<
+            "[add-column][int64] " <<
             "failed to add a column value: <" << value << ">");
     }
 
@@ -1985,7 +2015,8 @@ namespace grnarrow {
       check(ctx_,
             status,
             context <<
-            "[arrow][stream-writer][add-column][uint64] " <<
+            tag_ <<
+            "[add-column][uint64] " <<
             "failed to add a column value: <" << value << ">");
     }
 
@@ -2001,7 +2032,8 @@ namespace grnarrow {
       check(ctx_,
             status,
             context <<
-            "[arrow][stream-writer][add-column][float32] " <<
+            tag_ <<
+            "[add-column][float32] " <<
             "failed to add a column value: <" << value << ">");
     }
 
@@ -2017,7 +2049,8 @@ namespace grnarrow {
       check(ctx_,
             status,
             context <<
-            "[arrow][stream-writer][add-column][float] " <<
+            tag_ <<
+            "[add-column][float] " <<
             "failed to add a column value: <" << value << ">");
     }
 
@@ -2033,7 +2066,8 @@ namespace grnarrow {
       check(ctx_,
             status,
             context <<
-            "[arrow][stream-writer][add-column][timestamp] " <<
+            tag_ <<
+            "[add-column][timestamp] " <<
             "failed to add a column value: <" <<
             (value.tv_sec + (value.tv_nsec / GRN_TIME_NSEC_PER_SEC_F)) <<
             ">");
@@ -2051,7 +2085,8 @@ namespace grnarrow {
       check(ctx_,
             status,
             context <<
-            "[arrow][stream-writer][add-column][double] " <<
+            tag_ <<
+            "[add-column][double] " <<
             "failed to add a column value: <" <<
             value <<
             ">");
@@ -2087,7 +2122,8 @@ namespace grnarrow {
       check(ctx_,
             status,
             context <<
-            "[arrow][stream-writer][add-column][record] " <<
+            tag_ <<
+            "[add-column][record] " <<
             "failed to add a column value: <" <<
             arrow::util::string_view(GRN_TEXT_VALUE(&inspected),
                                      GRN_TEXT_LEN(&inspected)) <<
@@ -2148,7 +2184,8 @@ namespace grnarrow {
       check(ctx_,
             status,
             context <<
-            "[arrow][stream-writer][add-column][uvector] " <<
+            tag_ <<
+            "[add-column][uvector] " <<
             "failed to add a column value: <" <<
             arrow::util::string_view(GRN_TEXT_VALUE(&inspected),
                                      GRN_TEXT_LEN(&inspected)) <<
@@ -2165,19 +2202,16 @@ namespace grnarrow {
       auto status = record_batch_builder_->Flush(&record_batch);
       if (check(ctx_,
                 status,
-                "[arrow][stream-writer][flush] failed to flush record batch")) {
+                tag_ + "[flush] failed to flush record batch")) {
         status = writer_->WriteRecordBatch(*record_batch);
         check(ctx_,
               status,
-              "[arrow][stream-writer][flush] "
-              "failed to write flushed record batch");
+              tag_ + "[flush] failed to write flushed record batch");
       }
 
       auto n_fields = record_batch_builder_->num_fields();
       for (int i = 0; i < n_fields; ++i) {
-        auto builder = record_batch_builder_->GetField(i);
-        auto visitor = ArrayBuilderResetFullVisitor(ctx_, builder);
-        builder->type()->Accept(&visitor);
+        reset_full(record_batch_builder_->GetField(i));
       }
 
       n_records_ = 0;
@@ -2193,6 +2227,40 @@ namespace grnarrow {
     size_t n_records_;
     int current_column_index_;
     ObjectCache object_cache_;
+    std::string tag_;
+
+    // We need to execute ResetFull() with an appropriate dictionary length
+    // in order to avoid memory exhaustion and so on.
+    // - Dictionary data in DictionaryBuilder are held in memory until
+    //   ResetFull() is executed.
+    // - Some Apache Arrow implementations (clients) have a 2GB limitation on
+    //   the value size of array: C#.
+    void reset_full(arrow::ArrayBuilder *builder) {
+      switch (builder->type()->id()) {
+      case arrow::Type::DICTIONARY :
+        {
+          const int64_t dictionary_length_threshold = 10000;
+
+          // The value type of Dictionary is always string for now.
+          auto dictionary_builder =
+            static_cast<arrow::StringDictionaryBuilder *>(builder);
+          if (dictionary_builder->dictionary_length() >
+              dictionary_length_threshold) {
+            dictionary_builder->ResetFull();
+          }
+        }
+        break;
+      case arrow::Type::LIST :
+        {
+          auto list_builder = static_cast<arrow::ListBuilder *>(builder);
+          auto value_builder = list_builder->value_builder();
+          reset_full(value_builder);
+        }
+        break;
+      default :
+        break;
+      }
+    }
   };
 }
 
