@@ -1,6 +1,6 @@
 /*
-  Copyright(C) 2018  Brazil
-  Copyright(C) 2020-2021  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2018  Brazil
+  Copyright (C) 2020-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,7 @@ namespace grn {
     template <typename NUMERIC, typename FLOAT>
     NUMERIC
     cast_value(const char *raw_value) {
-      return *reinterpret_cast<const FLOAT *>(raw_value);
+      return static_cast<NUMERIC>(*reinterpret_cast<const FLOAT *>(raw_value));
     }
 
     template <>
@@ -73,28 +73,36 @@ namespace grn {
         value = *reinterpret_cast<const grn_bool *>(raw_value);
         break;
       case GRN_DB_INT8 :
-        value = *reinterpret_cast<const int8_t *>(raw_value);
+        value =
+          static_cast<NUMERIC>(*reinterpret_cast<const int8_t *>(raw_value));
         break;
       case GRN_DB_UINT8 :
-        value = *reinterpret_cast<const uint8_t *>(raw_value);
+        value =
+          static_cast<NUMERIC>(*reinterpret_cast<const uint8_t *>(raw_value));
         break;
       case GRN_DB_INT16 :
-        value = *reinterpret_cast<const int16_t *>(raw_value);
+        value =
+          static_cast<NUMERIC>(*reinterpret_cast<const int16_t *>(raw_value));
         break;
       case GRN_DB_UINT16 :
-        value = *reinterpret_cast<const uint16_t *>(raw_value);
+        value =
+          static_cast<NUMERIC>(*reinterpret_cast<const uint16_t *>(raw_value));
         break;
       case GRN_DB_INT32 :
-        value = *reinterpret_cast<const int32_t *>(raw_value);
+        value =
+          static_cast<NUMERIC>(*reinterpret_cast<const int32_t *>(raw_value));
         break;
       case GRN_DB_UINT32 :
-        value = *reinterpret_cast<const uint32_t *>(raw_value);
+        value =
+          static_cast<NUMERIC>(*reinterpret_cast<const uint32_t *>(raw_value));
         break;
       case GRN_DB_INT64 :
-        value = *reinterpret_cast<const int64_t *>(raw_value);
+        value =
+          static_cast<NUMERIC>(*reinterpret_cast<const int64_t *>(raw_value));
         break;
       case GRN_DB_UINT64 :
-        value = *reinterpret_cast<const uint64_t *>(raw_value);
+        value =
+          static_cast<NUMERIC>(*reinterpret_cast<const uint64_t *>(raw_value));
         break;
       case GRN_DB_FLOAT32 :
         value = cast_value<NUMERIC, float>(raw_value);
@@ -214,18 +222,18 @@ extern "C" {
   static uint32_t
   grn_uvector_element_size_internal(grn_ctx *ctx, grn_obj *uvector)
   {
-    uint32_t element_size = grn_type_id_size(ctx, uvector->header.domain);
+    size_t element_size = grn_type_id_size(ctx, uvector->header.domain);
     if (grn_obj_is_weight_uvector(ctx, uvector)) {
       element_size += sizeof(float);
     }
-    return element_size;
+    return static_cast<uint32_t>(element_size);
   }
 
   static uint32_t
   grn_uvector_size_internal(grn_ctx *ctx, grn_obj *uvector)
   {
     uint32_t element_size = grn_uvector_element_size_internal(ctx, uvector);
-    return GRN_BULK_VSIZE(uvector) / element_size;
+    return static_cast<uint32_t>(GRN_BULK_VSIZE(uvector) / element_size);
   }
 
   uint32_t
@@ -239,13 +247,13 @@ extern "C" {
     GRN_API_ENTER;
     switch (vector->header.type) {
     case GRN_BULK :
-      size = GRN_BULK_VSIZE(vector);
+      size = static_cast<uint32_t>(GRN_BULK_VSIZE(vector));
       break;
     case GRN_UVECTOR :
       size = grn_uvector_size_internal(ctx, vector);
       break;
     case GRN_PVECTOR :
-      size = GRN_BULK_VSIZE(vector) / sizeof(grn_obj *);
+      size = static_cast<uint32_t>(GRN_BULK_VSIZE(vector) / sizeof(grn_obj *));
       break;
     case GRN_VECTOR :
       size = vector->u.v.n_sections;
@@ -304,7 +312,7 @@ extern "C" {
                                                    &weight_float,
                                                    domain);
     if (weight) {
-      *weight = weight_float;
+      *weight = static_cast<uint32_t>(weight_float);
     }
     return length;
   }
@@ -353,7 +361,7 @@ extern "C" {
                                                    &weight_float,
                                                    domain);
     if (weight) {
-      *weight = weight_float;
+      *weight = static_cast<uint32_t>(weight_float);
     }
     return length;
   }
@@ -411,7 +419,7 @@ extern "C" {
       grn_obj *body = grn_vector_body(ctx, v);
       grn_section *vp = &v->u.v.sections[v->u.v.n_sections];
       vp->offset = v->u.v.n_sections ? vp[-1].offset + vp[-1].length : 0;
-      vp->length = GRN_BULK_VSIZE(body) - vp->offset;
+      vp->length = static_cast<uint32_t>(GRN_BULK_VSIZE(body) - vp->offset);
       vp->weight = weight;
       vp->domain = domain;
     }
@@ -443,7 +451,9 @@ extern "C" {
         if (flags & GRN_VECTOR_PACK_WEIGHT_FLOAT32) {
           GRN_FLOAT32_PUT(ctx, footer, section->weight);
         } else {
-          grn_text_benc(ctx, footer, section->weight);
+          grn_text_benc(ctx,
+                        footer,
+                        static_cast<unsigned int>(section->weight));
         }
         grn_text_benc(ctx, footer, section->domain);
       }
@@ -474,7 +484,7 @@ extern "C" {
     }
     {
       grn_obj *body = grn_vector_body(ctx, vector);
-      uint32_t offset = GRN_BULK_VSIZE(body);
+      uint32_t offset = static_cast<uint32_t>(GRN_BULK_VSIZE(body));
       uint32_t o = 0;
       for (uint32_t i = 0; i < n; ++i) {
         grn_section *section = vector->u.v.sections + n0 + i;
@@ -498,7 +508,9 @@ extern "C" {
             grn_memcpy(&(section->weight), p, sizeof(float));
             p += sizeof(float);
           } else {
-            GRN_B_DEC(section->weight, p);
+            uint32_t weight;
+            GRN_B_DEC(weight, p);
+            section->weight = static_cast<float>(weight);
           }
           GRN_B_DEC(section->domain, p);
         }
@@ -520,7 +532,7 @@ extern "C" {
                                         vector,
                                         str,
                                         str_len,
-                                        weight,
+                                        static_cast<float>(weight),
                                         domain);
   }
 
@@ -574,6 +586,58 @@ extern "C" {
     GRN_API_RETURN(ctx->rc);
   }
 
+  grn_obj *
+  grn_vector_join(grn_ctx *ctx,
+                  grn_obj *vector,
+                  const char *separator,
+                  int separator_length,
+                  grn_obj *destination)
+  {
+    GRN_API_ENTER;
+    if (!vector) {
+      ERR(GRN_INVALID_ARGUMENT, "[vector][join] vector is NULL");
+      GRN_API_RETURN(NULL);
+    }
+    if (vector->header.type != GRN_VECTOR) {
+      grn_obj type_name;
+      GRN_TEXT_INIT(&type_name, 0);
+      grn_inspect_type(ctx, &type_name, vector->header.type);
+      ERR(GRN_INVALID_ARGUMENT, "[vector][join] must be GRN_UVECTOR: %.*s",
+          (int)GRN_TEXT_LEN(&type_name), GRN_TEXT_VALUE(&type_name));
+      GRN_OBJ_FIN(ctx, &type_name);
+      GRN_API_RETURN(NULL);
+    }
+    if (separator_length < 0) {
+      separator_length = (int)strlen(separator);
+    }
+    if (!destination) {
+      destination = grn_obj_open(ctx, GRN_BULK, 0, GRN_DB_TEXT);
+    }
+    uint32_t n_elements = grn_vector_size(ctx, vector);
+    uint32_t i;
+    grn_obj element;
+    GRN_TEXT_INIT(&element, GRN_OBJ_DO_SHALLOW_COPY);
+    for (i = 0; i < n_elements; i++) {
+      if (i > 0 && separator_length > 0) {
+        GRN_TEXT_PUT(ctx, destination, separator, separator_length);
+      }
+      const char *content;
+      grn_id domain;
+      uint32_t content_size = grn_vector_get_element_float(ctx,
+                                                           vector,
+                                                           i,
+                                                           &content,
+                                                           NULL,
+                                                           &domain);
+      GRN_TEXT_SET(ctx, &element, content, content_size);
+      element.header.domain = domain;
+      grn_obj_cast(ctx, &element, destination, false);
+      element.header.domain = GRN_DB_TEXT;
+    }
+    GRN_OBJ_FIN(ctx, &element);
+    GRN_API_RETURN(destination);
+  }
+
   uint32_t
   grn_uvector_size(grn_ctx *ctx, grn_obj *uvector)
   {
@@ -594,7 +658,7 @@ extern "C" {
 
     GRN_API_ENTER;
     size_t size = grn_uvector_size_internal(ctx, uvector);
-    GRN_API_RETURN(size);
+    GRN_API_RETURN(static_cast<uint32_t>(size));
   }
 
   uint32_t
@@ -628,7 +692,10 @@ extern "C" {
                           grn_id id,
                           uint32_t weight)
   {
-    return grn_uvector_add_element_record(ctx, uvector, id, weight);
+    return grn_uvector_add_element_record(ctx,
+                                          uvector,
+                                          id,
+                                          static_cast<float>(weight));
   }
 
   grn_rc
@@ -665,7 +732,7 @@ extern "C" {
                                                offset,
                                                &weight_float);
     if (weight) {
-      *weight = weight_float;
+      *weight = static_cast<uint32_t>(weight_float);
     }
     return id;
   }
@@ -752,4 +819,95 @@ extern "C" {
     grn_bulk_write(ctx, dest, GRN_BULK_HEAD(src), GRN_BULK_VSIZE(src));
     GRN_API_RETURN(ctx->rc);
   }
+
+  grn_obj *
+  grn_uvector_join(grn_ctx *ctx,
+                   grn_obj *uvector,
+                   const char *separator,
+                   int separator_length,
+                   grn_obj *destination)
+  {
+    GRN_API_ENTER;
+    if (!uvector) {
+      ERR(GRN_INVALID_ARGUMENT, "[uvector][join] uvector is NULL");
+      GRN_API_RETURN(NULL);
+    }
+    if (uvector->header.type != GRN_UVECTOR) {
+      grn_obj type_name;
+      GRN_TEXT_INIT(&type_name, 0);
+      grn_inspect_type(ctx, &type_name, uvector->header.type);
+      ERR(GRN_INVALID_ARGUMENT, "[uvector][join] must be GRN_UVECTOR: %.*s",
+          (int)GRN_TEXT_LEN(&type_name), GRN_TEXT_VALUE(&type_name));
+      GRN_OBJ_FIN(ctx, &type_name);
+      GRN_API_RETURN(NULL);
+    }
+    if (separator_length < 0) {
+      separator_length = (int)strlen(separator);
+    }
+    if (!destination) {
+      destination = grn_obj_open(ctx, GRN_BULK, 0, GRN_DB_TEXT);
+    }
+    uint32_t element_size = grn_uvector_element_size_internal(ctx, uvector);
+    uint32_t element_content_size = element_size;
+    if (grn_obj_is_weight_uvector(ctx, uvector)) {
+      element_content_size -= sizeof(float);
+    }
+    uint32_t n_elements = grn_uvector_size_internal(ctx, uvector);
+    uint32_t i;
+    grn_obj element;
+    GRN_VALUE_FIX_SIZE_INIT(&element,
+                            GRN_OBJ_DO_SHALLOW_COPY,
+                            uvector->header.domain);
+    for (i = 0; i < n_elements; i++) {
+      if (i > 0 && separator_length > 0) {
+        GRN_TEXT_PUT(ctx, destination, separator, separator_length);
+      }
+      GRN_TEXT_SET(ctx,
+                   &element,
+                   GRN_BULK_HEAD(uvector) + (element_size * i),
+                   element_content_size);
+      grn_obj_cast(ctx, &element, destination, false);
+    }
+    GRN_OBJ_FIN(ctx, &element);
+    GRN_API_RETURN(destination);
+  }
+
+  grn_obj *
+  grn_pvector_join(grn_ctx *ctx,
+                   grn_obj *pvector,
+                   const char *separator,
+                   int separator_length,
+                   grn_obj *destination)
+  {
+    GRN_API_ENTER;
+    if (!pvector) {
+      ERR(GRN_INVALID_ARGUMENT, "[pvector][join] pvector is NULL");
+      GRN_API_RETURN(NULL);
+    }
+    if (pvector->header.type != GRN_PVECTOR) {
+      grn_obj type_name;
+      GRN_TEXT_INIT(&type_name, 0);
+      grn_inspect_type(ctx, &type_name, pvector->header.type);
+      ERR(GRN_INVALID_ARGUMENT, "[pvector][join] must be GRN_PVECTOR: %.*s",
+          (int)GRN_TEXT_LEN(&type_name), GRN_TEXT_VALUE(&type_name));
+      GRN_OBJ_FIN(ctx, &type_name);
+      GRN_API_RETURN(NULL);
+    }
+    if (separator_length < 0) {
+      separator_length = (int)strlen(separator);
+    }
+    if (!destination) {
+      destination = grn_obj_open(ctx, GRN_BULK, 0, GRN_DB_TEXT);
+    }
+    size_t n_elements = GRN_PTR_VECTOR_SIZE(pvector);
+    size_t i;
+    for (i = 0; i < n_elements; i++) {
+      if (i > 0 && separator_length > 0) {
+        GRN_TEXT_PUT(ctx, destination, separator, separator_length);
+      }
+      grn_obj *element = GRN_PTR_VALUE_AT(pvector, i);
+      grn_obj_cast(ctx, element, destination, false);
+    }
+    GRN_API_RETURN(destination);
+  }
 }
